package cn.iocoder.yudao.module.bpm.controller.admin.definition;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyExcelVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyExportReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyUpdateReqVO;
import cn.iocoder.yudao.module.bpm.convert.definition.ProcessCopyConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.definition.ProcessCopyDO;
import cn.iocoder.yudao.module.bpm.service.definition.ProcessCopyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

/**
 * @author Administrator
 */
@Api(tags = "管理后台 - 流程抄送")
@RestController
@RequestMapping("/bpm/process-copy")
@Validated
public class ProcessCopyController {

    @Resource
    private ProcessCopyService processCopyService;

    @PostMapping("/create")
    @ApiOperation("创建流程抄送")
    @PreAuthorize("@ss.hasPermission('bpm:process-copy:create')")
    public CommonResult<Long> createProcessCopy(@Valid @RequestBody ProcessCopyCreateReqVO createReqVO) {
        return success(processCopyService.createProcessCopy(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新流程抄送")
    @PreAuthorize("@ss.hasPermission('bpm:process-copy:update')")
    public CommonResult<Boolean> updateProcessCopy(@Valid @RequestBody ProcessCopyUpdateReqVO updateReqVO) {
        processCopyService.updateProcessCopy(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除流程抄送")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('bpm:process-copy:delete')")
    public CommonResult<Boolean> deleteProcessCopy(@RequestParam("id") Long id) {
        processCopyService.deleteProcessCopy(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得流程抄送")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('bpm:process-copy:query')")
    public CommonResult<ProcessCopyRespVO> getProcessCopy(@RequestParam("id") Long id) {
        ProcessCopyDO processCopy = processCopyService.getProcessCopy(id);
        return success(ProcessCopyConvert.INSTANCE.convert(processCopy));
    }

    @GetMapping("/list")
    @ApiOperation("获得流程抄送列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('bpm:process-copy:query')")
    public CommonResult<List<ProcessCopyRespVO>> getProcessCopyList(@RequestParam("ids") Collection<Long> ids) {
        List<ProcessCopyDO> list = processCopyService.getProcessCopyList(ids);
        return success(ProcessCopyConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得流程抄送分页")
    @PreAuthorize("@ss.hasPermission('bpm:process-copy:query')")
    public CommonResult<PageResult<ProcessCopyRespVO>> getProcessCopyPage(@Valid ProcessCopyPageReqVO pageVO) {
        PageResult<ProcessCopyDO> pageResult = processCopyService.getProcessCopyPage(pageVO);
        return success(ProcessCopyConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出流程抄送 Excel")
    @PreAuthorize("@ss.hasPermission('bpm:process-copy:export')")
    @OperateLog(type = EXPORT)
    public void exportProcessCopyExcel(@Valid ProcessCopyExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<ProcessCopyDO> list = processCopyService.getProcessCopyList(exportReqVO);
        // 导出 Excel
        List<ProcessCopyExcelVO> datas = ProcessCopyConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "流程抄送.xls", "数据", ProcessCopyExcelVO.class, datas);
    }

}
