package cn.iocoder.yudao.module.bpm.service.definition;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyExportReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyUpdateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.definition.ProcessCopyDO;
import org.flowable.engine.runtime.ProcessInstance;

/**
 * 流程抄送 Service 接口
 *
 * @author 系统管理员
 */
public interface ProcessCopyService extends IService<ProcessCopyDO> {

    /**
     * 创建流程抄送
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createProcessCopy(@Valid ProcessCopyCreateReqVO createReqVO);

    /**
     * 更新流程抄送
     *
     * @param updateReqVO 更新信息
     */
    void updateProcessCopy(@Valid ProcessCopyUpdateReqVO updateReqVO);

    /**
     * 删除流程抄送
     *
     * @param id 编号
     */
    void deleteProcessCopy(Long id);

    /**
     * 获得流程抄送
     *
     * @param id 编号
     * @return 流程抄送
     */
    ProcessCopyDO getProcessCopy(Long id);

    /**
     * 获得流程抄送列表
     *
     * @param ids 编号
     * @return 流程抄送列表
     */
    List<ProcessCopyDO> getProcessCopyList(Collection<Long> ids);

    /**
     * 获得流程抄送分页
     *
     * @param pageReqVO 分页查询
     * @return 流程抄送分页
     */
    PageResult<ProcessCopyDO> getProcessCopyPage(ProcessCopyPageReqVO pageReqVO);

    /**
     * 获得流程抄送列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 流程抄送列表
     */
    List<ProcessCopyDO> getProcessCopyList(ProcessCopyExportReqVO exportReqVO);

    /**
     * 抄送
     * @param vo
     * @return
     */
    void makeCopy(BpmTaskReqVO vo, ProcessInstance instance);

    /**
     * 抄送
     * @param vo
     * @return
     */
    void makeCopy(BpmTaskReqVO vo);
}
