package cn.iocoder.yudao.module.bpm.service.definition;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyExportReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.definition.vo.copy.ProcessCopyUpdateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReqVO;
import cn.iocoder.yudao.module.bpm.convert.definition.ProcessCopyConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.definition.ProcessCopyDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.definition.ProcessCopyMapper;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.runtime.ProcessInstance;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.PROCESS_COPY_NOT_EXISTS;

/**
 * 流程抄送 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ProcessCopyServiceImpl extends AbstractService<ProcessCopyMapper, ProcessCopyDO> implements ProcessCopyService {

    @Resource
    private ProcessCopyMapper processCopyMapper;

    @Resource
    private RuntimeService runtimeService;

    @Resource
    private AdminUserApi adminUserApi;

    @Override
    public Long createProcessCopy(ProcessCopyCreateReqVO createReqVO) {
        // 插入
        ProcessCopyDO processCopy = ProcessCopyConvert.INSTANCE.convert(createReqVO);
        processCopyMapper.insert(processCopy);
        // 返回
        return processCopy.getId();
    }

    @Override
    public void updateProcessCopy(ProcessCopyUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateProcessCopyExists(updateReqVO.getId());
        // 更新
        ProcessCopyDO updateObj = ProcessCopyConvert.INSTANCE.convert(updateReqVO);
        processCopyMapper.updateById(updateObj);
    }

    @Override
    public void deleteProcessCopy(Long id) {
        // 校验存在
        this.validateProcessCopyExists(id);
        // 删除
        processCopyMapper.deleteById(id);
    }

    private void validateProcessCopyExists(Long id) {
        if (processCopyMapper.selectById(id) == null) {
            throw exception(PROCESS_COPY_NOT_EXISTS);
        }
    }

    @Override
    public ProcessCopyDO getProcessCopy(Long id) {
        return processCopyMapper.selectById(id);
    }

    @Override
    public List<ProcessCopyDO> getProcessCopyList(Collection<Long> ids) {
        return processCopyMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ProcessCopyDO> getProcessCopyPage(ProcessCopyPageReqVO pageReqVO) {
        IPage<ProcessCopyDO> mpPage = MyBatisUtils.buildPage(pageReqVO);
        processCopyMapper.getProcessCopyDoPage(mpPage, pageReqVO);
        PageResult<ProcessCopyDO> pageResult = PageResult.of(mpPage);
        return pageResult;
        //return processCopyMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ProcessCopyDO> getProcessCopyList(ProcessCopyExportReqVO exportReqVO) {
        return processCopyMapper.selectList(exportReqVO);
    }

    @Override
    public void makeCopy(BpmTaskReqVO vo, ProcessInstance instance) {
        // 若抄送用户为空，则不需要处理，返回成功
        String[] copies = vo.getCopyUserIds();
        if (Objects.isNull(vo) || ArrayUtil.isEmpty(copies)) {
            return;
        }

        log.error("bmp333333333333333333333333333333");
        if (Objects.isNull(instance)) {
            log.error("bmp444444444444444444444444444444");
            return;
        }
        // 初始化集合
        List<ProcessCopyDO> copyList = new ArrayList<>(copies.length);
        // 获取用户列表
        AdminUserRespDTO dto = adminUserApi.getUser(Long.parseLong(instance.getStartUserId()));
        for (String id : copies) {
            log.error("bmp5555555555555555555555555555555");
            if (StrUtil.isBlank(id)) {
                continue;
            }

            ProcessCopyDO copy = new ProcessCopyDO();
            copy.setTitle(vo.getTaskName());
            copy.setProcessId(instance.getProcessDefinitionId());
            copy.setProcessName(instance.getProcessDefinitionName());
            copy.setDeploymentId(instance.getDeploymentId());
            copy.setInstanceId(vo.getInstanceId());
            copy.setTaskId(vo.getId());
            copy.setUserId(Long.valueOf(id));
            copy.setOriginatorId(Long.parseLong(instance.getStartUserId()));
            if (Objects.nonNull(dto)) {
                copy.setOriginatorName(dto.getNickname());
            }

            copyList.add(copy);
        }
        log.error("bmp66666666666666666666666666666");
        baseMapper.insertBatch(copyList);
    }

    @Override
    public void makeCopy(BpmTaskReqVO vo) {

        // 若抄送用户为空，则不需要处理，返回成功
        String[] copies = vo.getCopyUserIds();
        if (Objects.isNull(vo) || ArrayUtil.isEmpty(copies)) {
            return;
        }
        ProcessInstance instance = runtimeService.createProcessInstanceQuery().processInstanceId(vo.getInstanceId()).singleResult();
        if (Objects.isNull(instance)) {
            return;
        }
        log.error("bmp-333333333333333333333333333333");
        if (Objects.isNull(instance)) {
            log.error("bmp-444444444444444444444444444444");
            return;
        }
        // 初始化集合
        List<ProcessCopyDO> copyList = new ArrayList<>(copies.length);
        // 获取用户列表
        AdminUserRespDTO dto = adminUserApi.getUser(Long.parseLong(instance.getStartUserId()));
        for (String id : copies) {
            log.error("bmp-5555555555555555555555555555555");
            if (StrUtil.isBlank(id)) {
                continue;
            }

            ProcessCopyDO copy = new ProcessCopyDO();
            copy.setTitle(vo.getTaskName());
            copy.setProcessId(instance.getProcessDefinitionId());
            copy.setProcessName(instance.getProcessDefinitionName());
            copy.setDeploymentId(instance.getDeploymentId());
            copy.setInstanceId(vo.getInstanceId());
            copy.setTaskId(vo.getId());
            copy.setUserId(Long.valueOf(id));
            copy.setOriginatorId(Long.parseLong(instance.getStartUserId()));
            if (Objects.nonNull(dto)) {
                copy.setOriginatorName(dto.getNickname());
            }

            copyList.add(copy);
        }
        log.error("bmp-66666666666666666666666666666");
        baseMapper.insertBatch(copyList);

    }
}
